/*
 * Squawk messenger.
 * Copyright (C) 2019  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QAbstractListModel>

#include <shared/keyinfo.h>

namespace Models {

class Keys : public QAbstractListModel {
public:
    Keys(QObject *parent = nullptr);
    ~Keys();

    void addKey(const Shared::KeyInfo& info);
    void clear();

    QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const override;
    int rowCount(const QModelIndex& parent = QModelIndex()) const override;
    const Shared::KeyInfo& key(unsigned int i) const;

    QHash<int, QByteArray> roleNames() const override;
    QModelIndex index(int row, int column, const QModelIndex & parent) const override;

    std::list<Shared::KeyInfo> modifiedKeys() const;
    std::list<Shared::KeyInfo> finalKeys() const;
    void finalKeys(std::list<Shared::KeyInfo>& out) const;

    enum Roles {
        Label = Qt::UserRole + 1,
        FingerPrint,
        TrustLevel,
        LastInteraction,
        Dirty
    };

public slots:
    void revertKey(int index);
    void setTrustLevel(int index, Shared::TrustLevel level);

private:
    std::deque<Shared::KeyInfo*> keys;
    std::map<int, Shared::KeyInfo*> modified;

private:
    static const QHash<int, QByteArray> roles;
};

}
