/*
 * Squawk messenger.
 * Copyright (C) 2019  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "expandinglist.h"

#include <QApplication>

ExpandingList::ExpandingList(QWidget* parent) :
    QListView(parent),
    scrollBarExtent(qApp->style()->pixelMetric(QStyle::PM_ScrollBarExtent))
{
    setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
}

bool ExpandingList::hasHeightForWidth() const {
#if (QT_VERSION < QT_VERSION_CHECK(6, 2, 0))
    return
        QAbstractItemView::sizeAdjustPolicy() == QAbstractScrollArea::AdjustToContents &&
        scrollBarExtent > 0;
#else
    return false;
#endif
}

int ExpandingList::heightForWidth(int width) const {
    QAbstractItemModel* md = model();

    if (md == nullptr)
        return 0;
    if (md->rowCount() == 0)
        return 0;

    const int rowCount = md->rowCount();
    int height = 0;
    for (int i = 0; i < rowCount; i++)
        height += QListView::sizeHintForRow(i);

    height += frameWidth();
    int minWidth = sizeHintForColumn(0) + frameWidth() + 1;

    if (width <= minWidth)
        height += scrollBarExtent + 1;

    return height;
}
